# core Python packages
import logging


# third party packages


# django packages
from django.contrib.auth.forms import AuthenticationForm
from django.shortcuts import render, HttpResponseRedirect
from django.core.urlresolvers import reverse
from django.utils.timezone import utc
from django.contrib import auth
from django.contrib.auth.decorators import login_required


# local imports
import utils

# start a logger
logger = logging.getLogger('default')


# Create your views here.
@login_required(login_url='/login/')
def index(request):
    logger.debug('Core index requested')
    
    # TODO: if not is_staff, user goes to list of their requests only
    
    installed_modules = utils.installed_modules()
    
    logger.debug('Component modules detected:')
    [logger.debug('%s: %s' % (k, v)) for k, v in installed_modules.iteritems()]

    context = {
        'title': 'The Django-HCUP Hachoir: Core Index', 
        'djhcup_components': installed_modules
        }
    
    template = 'djhcup_core_index.html'
    return render(request, template, context)


def login(request, messages=[]):
    logger.debug('Login page requested.')
    form = AuthenticationForm(None, request.POST or None)
    
    if request.user.is_authenticated():
        # log them out before proceeding
        auth.logout(request)
        
    if request.POST:
        # process submitted credentials
        form = AuthenticationForm(None, data=request.POST)
        if form.is_valid():
            user = form.get_user()
            if user.is_active:
                # not a disabled account
                auth.login(request, user)
                nextpage = request.GET.get('next', reverse('index'))
                return HttpResponseRedirect(nextpage)
            else:
                messages.append(dict(
                    type='error',
                    content='Your account has been disabled. Please contact an administrator.'))
        else:
            messages.append(dict(
                type='error',
                content='Unable to log in. Please try again.'))
    
    context = {
        'title': 'The Django-HCUP Hachoir: Login', 
        'form': form,
        'response_messages': messages
        }
    
    template = 'djhcup_login.html'
    return render(request, template, context)


def logout(request):
    logger.debug('Logout page requested.')
    messages = []
    
    if auth.logout(request):
        messages.append(dict(
            type='info',
            content='Successfully logged out.'
            ))
    
    return HttpResponseRedirect(reverse('login'))



from distutils.core import setup


setup(
    name='djhcup_core',
    version='0.20140801.0dev',
    include_package_data=True,
    description='A django-based interface for warehousing HCUP data',
    long_description=open('README.rst').read(),
    keywords='HCUP SAS healthcare analysis pandas',
    license='MIT',
    author='T.J. Biel',
    author_email='tbiel@med.umich.edu',
    packages=['djhcup_core'],
    provides=['djhcup_core'],
    classifiers=[
        'Framework :: Django',
        'License :: OSI Approved :: MIT License',
        'Topic :: Scientific/Engineering',
    ],
    package_data={'djhcup_core': [
                    'templates/*.*',
                    ]
                    },
    requires=[
        'django (>= 1.6)',
        'celery (>= 3.1.0)',
        'pandas (>= 0.11.0)',
        'pyhcup (>= 0.1.6)'
    ],
)


# core Python packages
import importlib
import string
from random import choice


# django imports
from django.conf import settings


# third party imports
import psycopg2


def installed_modules():
    """Check for installed modules and return list with relevant data
    """
    # detect presence/absence of component modules
    module_names = [
        'djhcup_core',
        'djhcup_staging',
        'djhcup_integration',
        'djhcup_reporting',
    ]
    installed = {}

    for x in module_names:
        # TODO: Rewrite to pull version numbers for each of these as well
        try:
            module = importlib.import_module(x)
            installed[x] = True
        except ImportError:
            installed[x] = False
    
    if len(installed) > 0:
        return installed
    else:
        return False


def get_pgcnxn(read_only=False):
    DB_DEF = settings.DATABASES['djhcup']
    cnxn = psycopg2.connect(
        host=DB_DEF['HOST'],
        port=DB_DEF['PORT'],
        user=DB_DEF['USER'],
        password=DB_DEF['PASSWORD'],
        database=DB_DEF['NAME'],
        )
    return cnxn


SALT_CHOICES = string.lowercase + string.digits
def salt(length=20, choices=SALT_CHOICES):
    return "".join(choice(choices) for n in xrange(length))